#include <stdint.h> //      uint_fast8_t  uint_fast32_t
#include "AT91SAM7S64.h"


/*         */
/***********************************************************************************************************************/
#define SPI_IORESET_BIT  AT91C_PIO_PA23  // DDS (MASTER RESET)
#define SPI_IOUPDATE_BIT AT91C_PIO_PA13  //    DDS (I/O_UPDATE)
#define SPI_CS_BIT       AT91C_PIO_PA14  //  DDS (CS)
#define	SPI_CLK_BIT		 AT91C_PIO_PA15  // SPI DDS (SCLK)
#define	SPI_DATA_BIT	 AT91C_PIO_PA16  // SPI DDS (SDIO_0)

#define prog_bit(bitv) prog_bit_impl((bitv))

#define SCLK_SET() do { AT91C_BASE_PIOA->PIO_SODR = SPI_CLK_BIT; } while (0)
#define SCLK_CLR() do { AT91C_BASE_PIOA->PIO_CODR = SPI_CLK_BIT; } while (0)

#define SDO_SET(val) do { \
	if (val) \
	AT91C_BASE_PIOA->PIO_SODR = SPI_DATA_BIT; \
	else \
	AT91C_BASE_PIOA->PIO_CODR = SPI_DATA_BIT; \
	} while (0)

#define prog_bit_impl(bitv) \
do { \
	SDO_SET(bitv);	/*    */ \
	SCLK_CLR(); \
	SCLK_SET();	\
 } while (0)

#define RAMFUNC_NONILINE __attribute__((__section__(".data_ramfunc"), noinline))
#define CPU_FREQ 48000000UL		/*       */
unsigned (enableIRQ)(void);
unsigned (disableIRQ)(void);
void prog_val_impl (unsigned char value, uint_fast8_t n);

// 
volatile uint_fast8_t freq_value_byte;         //      LPT
volatile uint_fast8_t adr_value_byte;          //      LPT
uint_fast8_t n = 8;                            //  
volatile uint_fast8_t pll;                     //    PLL DDS
volatile uint_fast32_t freq;                   // 32-      AD9958
volatile unsigned char k;                      //   C3
unsigned char value;                           //,      AD9958

/*************************************************************************************************************************/

/*     96     12     
  48 
 */
static void lowlevel_init_pll_clock_48_xtal12(void)
{
	enum { osc_mul = 8, osc_div = 1 };	// 12 MHz / 1 * 8 = 96 MHz

	// before reprogramming - set safe waitstates
	AT91C_BASE_MC->MC_FMR = AT91C_MC_FWS_1FWS;

	///////////////////////////////////////////////////////////////////////////
	// Init PMC Step 1. Enable Main Oscillator
	// Main Oscillator startup time is board specific:
	// Main Oscillator Startup Time worst case (3MHz) corresponds to 15ms
	// (0x40 for AT91C_CKGR_OSCOUNT field)
	///////////////////////////////////////////////////////////////////////////
	AT91C_BASE_PMC->PMC_MOR = (((AT91C_CKGR_OSCOUNT & (0x40 << 8)) | AT91C_CKGR_MOSCEN));
	// Wait Main Oscillator stabilization
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MOSCS))
		;
	///////////////////////////////////////////////////////////////////////////
	// Init PMC Step 2.
	// Set PLL to 96MHz (96,109MHz) and UDP Clock to 48MHz
	// PLL Startup time depends on PLL RC filter: worst case is choosen
	// UDP Clock (48,058MHz) is compliant with the Universal Serial Bus
	// Specification (+/- 0.25% for full speed)
	///////////////////////////////////////////////////////////////////////////
	AT91C_BASE_PMC->PMC_PLLR = 
						AT91C_CKGR_USBDIV_1 |
						AT91C_CKGR_OUT_0 |
						(AT91C_CKGR_PLLCOUNT & (16 << 8)) |		// PLL lock signalling delay
						(AT91C_CKGR_MUL & ((osc_mul - 1) << 16)) |
						(AT91C_CKGR_DIV & osc_div);		
	
	// Wait for PLL stabilization
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCK))
		;
	
	// Wait until the master clock is established for the case we already
	// turn on the PLL
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY))
		;

	///////////////////////////////////////////////////////////////////////////
	// Init PMC Step 3.
	// Selection of Master Clock MCK equal to (Processor Clock PCK) PLL/2=48MHz
	// The PMC_MCKR register must not be programmed in a single write operation
	// (see. Product Errata Sheet)
	///////////////////////////////////////////////////////////////////////////
	AT91C_BASE_PMC->PMC_MCKR = AT91C_PMC_PRES_CLK_2;	// 96 / 2 = 48

	// Wait until the master clock is established
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY))
		;

	AT91C_BASE_PMC->PMC_MCKR = (AT91C_PMC_PRES_CLK_2 | AT91C_PMC_CSS_PLL_CLK);

	// Wait until the master clock is established
	while (!(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY))
		;

	// as final stage - set desired waitstates
	AT91C_BASE_MC->MC_FMR = AT91C_MC_FWS_1FWS; // 1 Wait State to work at 48 MHz
}

void arm_cpu_initialize(void)
{

	// Disable Watchdog
	AT91C_BASE_WDTC->WDTC_WDMR = AT91C_WDTC_WDDIS;

	// Enable NRST input.     .
	AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (AT91C_RSTC_KEY & (0xA5UL << 24));

	// init clock sources and memory timings
	// use one of alternatives
	//
	lowlevel_init_pll_clock_48_xtal12();
}



// for 48 MHz CPU clock, execute from RAM rom with 1 read wait state
//
void RAMFUNC_NONILINE _delay_us(int timeUS)
{
	// CPU_FREQ / 6500000 = 7
	const int top = timeUS * 25 / (CPU_FREQ / 6500000); 
	// todo: test
	//const int top = timeUS * 1625 / (CPU_FREQ / 100000); 
	volatile int n;
	for (n = 0; n < top; ++ n)
	{
	}
}
// exactly as required
//
void _delay_ms(int timeMS)
{
	volatile int n;
	for (n = 0; n < timeMS; ++ n)
	{
		_delay_us(1000);
	}
}

#define IRQ_MASK 0x00000080
#define FIQ_MASK 0x00000040
#define INT_MASK (IRQ_MASK | FIQ_MASK)

static inline unsigned hw_get_cpsr(void)
{
	unsigned long retval;
	asm volatile (" mrs  %0, cpsr" : "=r" (retval) : /* no inputs */  );
	return retval;
}

static inline void hw_set_cpsr(unsigned val)
{
	asm volatile (" msr  cpsr, %0" : /* no outputs */ : "r" (val)  );
}

unsigned RAMFUNC_NONILINE (disableIRQ)(void)
{
	unsigned _cpsr;
	_cpsr = hw_get_cpsr();
	hw_set_cpsr(_cpsr | IRQ_MASK);
	return _cpsr;
}

unsigned RAMFUNC_NONILINE (restoreIRQ)(unsigned oldCPSR)
{
	unsigned _cpsr;

	_cpsr = hw_get_cpsr();
	hw_set_cpsr((_cpsr & ~IRQ_MASK) | (oldCPSR & IRQ_MASK));
	return _cpsr;
}

unsigned RAMFUNC_NONILINE (enableIRQ)(void)
{
	unsigned _cpsr;

	_cpsr = hw_get_cpsr();
	hw_set_cpsr(_cpsr & ~IRQ_MASK);
	return _cpsr;
}

unsigned RAMFUNC_NONILINE (disableFIQ)(void)
{
	unsigned _cpsr;

	_cpsr = hw_get_cpsr();
	hw_set_cpsr(_cpsr | FIQ_MASK);
	return _cpsr;
}

unsigned RAMFUNC_NONILINE (restoreFIQ)(unsigned oldCPSR)
{
	unsigned _cpsr;

	_cpsr = hw_get_cpsr();
	hw_set_cpsr((_cpsr & ~FIQ_MASK) | (oldCPSR & FIQ_MASK));
	return _cpsr;
}

unsigned RAMFUNC_NONILINE (enableFIQ)(void)
{
	unsigned _cpsr;

	_cpsr = hw_get_cpsr();
	hw_set_cpsr(_cpsr & ~FIQ_MASK);
	return _cpsr;
}

static void RAMFUNC_NONILINE AT91F_FIQHandler(void)
{
	for (;;)
	{
	}
}

//   ,    IVR (interrupt vector register) 
//   -    .
static void RAMFUNC_NONILINE AT91F_Spurious_handler(void)
{
	for (;;)
	{
	}
}
static void AT91F_DEF_IRQHandler(void) 
{
	for (;;)
	{
	}
}

/*    */
static void lowlevwl_interrupts_init(void)
{
	unsigned i;

	// Disable all interrupts
	AT91C_BASE_AIC->AIC_IDCR = 0xFFFFFFFF;
	// Clear all interrupts
	AT91C_BASE_AIC->AIC_ICCR = 0xFFFFFFFF;

	// Perform 8 IT acknoledge (write any value in EOICR)
	for (i = 0; i < 8 ; i++) {
		AT91C_BASE_AIC->AIC_EOICR = 0;
	}

	// Set up the default interrupts handler vectors
	AT91C_BASE_AIC->AIC_SVR [0] = (unsigned int) AT91F_FIQHandler;
	for (i=1;i < 31; i++)
	{
	    AT91C_BASE_AIC->AIC_SVR [i] = (unsigned int) AT91F_DEF_IRQHandler;
	}
	AT91C_BASE_AIC->AIC_SPU  = (unsigned int) AT91F_Spurious_handler;

	// Enable the Debug mode
	////AT91C_BASE_AIC->AIC_DCR = AT91C_AIC_DCR_PROT;
	
}


//  USB .
static void usb_disable(void)
{
	AT91C_BASE_PMC->PMC_PCER = 1UL << AT91C_ID_UDP; //   
	AT91C_BASE_UDP->UDP_TXVC = AT91C_UDP_TXVDIS;	// usp -
	AT91C_BASE_PMC->PMC_PCDR = 1UL << AT91C_ID_UDP; //   
	AT91C_BASE_PMC->PMC_SCDR = AT91C_PMC_UDP;
}


//                       
//enum { IPINS = (AT91C_PIO_PA20 | AT91C_PIO_PA30) }; //    (   )
//enum { C2_DATA = AT91C_PIO_PA30 };//       C2 (..  LPT    )
//enum { C3_ADR = AT91C_PIO_PA20 };//       C3 (..  LPT     AD9854)

// ,     PIO_ISR,     PIO  
//static RAMFUNC_NONILINE void AT91F_PIOA_IRQHandler(void) 
//{           
   //if ((AT91C_BASE_PIOA->PIO_PDSR & (C3_ADR)) == 0) //  "0"  PA20 ( C3),    LPT        
   //{                                 
   //++ k ;                    //    C3
      //if (k == 2)            //    ,          
      //{                                                                                           
        //   LPT           adr_value_byte                                            
        //const unsigned long inval = AT91C_BASE_PIOA->PIO_PDSR;
        //adr_value_byte =
        //(inval & 0x0f) |
        //((inval & (1UL << 29)) ? 0x10 : 0x00) |
        //((inval & (1UL << 28)) ? 0x20 : 0x00) ;                                   
        //       ,         , 
        // ..       AD9854
        //if (adr_value_byte == 0x04)
        //{                                  
        //freq = (freq_value_byte << 24);              
        //} 
        //if (adr_value_byte == 0x05)
        //{                                    
        //freq = freq + (freq_value_byte << 16);        
        //}           
        //if (adr_value_byte == 0x06)
        //{                                    
        //freq = freq + (freq_value_byte << 8);                    
        //}                     
        //if (adr_value_byte == 0x07)
        //{                              
        //freq = freq + (freq_value_byte << 0);//           AD9958
        //freq = freq * 2 / 5;                 //         AD9958  AD9854
        //prog_val_impl(0x04, n);                        //    FTW0   AD9958            
        //prog_val_impl((freq >> 24), n);                //     
        //prog_val_impl((freq >> 16), n);                //     
        //prog_val_impl((freq >> 8), n);                 //     
        //prog_val_impl((freq >> 0), n);                 //                        
        //AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT;  //     
        //_delay_us(1);                   
        //AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT;  //           
        //}        
        //   PLL      255...500 
        //if (adr_value_byte == 0x1E)
        //{
        //pll = 0; 
        //pll = (freq_value_byte << 2);//  LPT .    2               
        //pll = pll + 0x80;       //    255...500                                        
        //prog_val_impl(0x01, n); //    AD9958   FR1                    
        //prog_val_impl(pll, n);  //    AD9958  ,    PLL   VCO gain control 
        //prog_val_impl(0x00, n); //      AD9958    
        //prog_val_impl(0x00, n); //      AD9958    
        //AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
        //_delay_us(1);
        //AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //            
        //}                                                
      //}            
   //     3
   //}
   
   /*       2 */             
   //if ((AT91C_BASE_PIOA->PIO_PDSR & (C2_DATA)) != 0)  //  "1"   PA30 ( C2),    LPT                             
   //{                                       
   //const unsigned long inpval = AT91C_BASE_PIOA->PIO_PDSR;                                
   //freq_value_byte = 
   //(inpval & 0x0f) |
   //((inpval & (1UL << 29)) ? 0x10 : 0x00) |
   //((inpval & (1UL << 28)) ? 0x20 : 0x00) |
   //((inpval & (1UL << 27)) ? 0x40 : 0x00) |
   //((inpval & (1UL << 4)) ? 0x80 : 0x00) ;
   //k = 0;  //     - C3
   //     2                               
   //} 
   //(void) AT91C_BASE_PIOA->PIO_ISR; //      PIO    
//}


RAMFUNC_NONILINE void AT91F_IRQ0_Handler(void) //      C3
{  
   ++ k ;                    //   C3
      if (k == 2)            //   ,          
      {                                                                                           
        //   LPT          adr_value_byte                                            
        const unsigned long inval = AT91C_BASE_PIOA->PIO_PDSR;
        adr_value_byte =
        (inval & 0x0f) |
        ((inval & (1UL << 29)) ? 0x10 : 0x00) |
        ((inval & (1UL << 28)) ? 0x20 : 0x00) ;                                   
        //       ,         , 
        // ..       AD9854
        if (adr_value_byte == 0x04)
        {                                  
        freq = (freq_value_byte << 24);              
        } 
        if (adr_value_byte == 0x05)
        {                                    
        freq = freq + (freq_value_byte << 16);        
        }           
        if (adr_value_byte == 0x06)
        {                                    
        freq = freq + (freq_value_byte << 8);                    
        }                     
        if (adr_value_byte == 0x07)
        {                              
        freq = freq + (freq_value_byte << 0);//           AD9958
        freq = freq * 2 / 5;                 //         AD9958  AD9854
        prog_val_impl(0x04, n);                        //    FTW0   AD9958            
        prog_val_impl((freq >> 24), n);                //     
        prog_val_impl((freq >> 16), n);                //     
        prog_val_impl((freq >> 8), n);                 //     
        prog_val_impl((freq >> 0), n);                 //                        
        AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT;  //     
        _delay_us(1);                   
        AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT;  //           
        }        
        //   PLL      255...500 
        if (adr_value_byte == 0x1E)
        {
        pll = 0; 
        pll = (freq_value_byte << 2);//  LPT .    2               
        pll = pll + 0x80;       //    255...500                                        
        prog_val_impl(0x01, n); //    AD9958   FR1                    
        prog_val_impl(pll, n);  //    AD9958  ,    PLL   VCO gain control 
        prog_val_impl(0x00, n); //      AD9958    
        prog_val_impl(0x00, n); //      AD9958    
        AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
        _delay_us(1);
        AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //            
        }                                                
      }                                                                                                   
   AT91C_BASE_AIC->AIC_ICCR = (1UL << AT91C_ID_IRQ0); //    
}


RAMFUNC_NONILINE void AT91F_IRQ1_Handler(void) //      C2
{                                       
   const unsigned long inpval = AT91C_BASE_PIOA->PIO_PDSR;                                
   freq_value_byte = 
   (inpval & 0x0f) |
   ((inpval & (1UL << 29)) ? 0x10 : 0x00) |
   ((inpval & (1UL << 28)) ? 0x20 : 0x00) |
   ((inpval & (1UL << 27)) ? 0x40 : 0x00) |
   ((inpval & (1UL << 4)) ? 0x80 : 0x00) ;
   k = 0;  //    - C3                                   
   AT91C_BASE_AIC->AIC_ICCR = (1UL << AT91C_ID_IRQ1); //    
}    


void irq0_initialize(void) //  IRQ0 ( PA20,  C3)
{
    //   IRQ0
    //AT91C_BASE_PMC->PMC_PCER = (1UL << AT91C_ID_IRQ0) | (1UL << AT91C_ID_PIOA); 
    AT91C_BASE_PMC->PMC_PCER = (1UL << AT91C_ID_IRQ0);

    // IRQ  
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA20_IRQ0; //  PA20   PIO
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA20_IRQ0; // I/O    B
    
    //   SYS
    AT91C_BASE_AIC->AIC_IDCR = (1UL << AT91C_ID_IRQ0);  // 
    AT91C_BASE_AIC->AIC_SVR [AT91C_ID_IRQ0] = (AT91_REG) AT91F_IRQ0_Handler; //   
//AT91C_BASE_AIC->AIC_SMR [AT91C_ID_IRQ0] = (AT91C_AIC_SRCTYPE & AT91C_AIC_SRCTYPE_POSITIVE_EDGE) | (AT91C_AIC_PRIOR & AT91C_AIC_PRIOR_HIGHEST);
AT91C_BASE_AIC->AIC_SMR [AT91C_ID_IRQ0] = (AT91C_AIC_SRCTYPE & AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE) | (AT91C_AIC_PRIOR & AT91C_AIC_PRIOR_HIGHEST);   
    AT91C_BASE_AIC->AIC_ICCR = (1UL << AT91C_ID_IRQ0); // 
    AT91C_BASE_AIC->AIC_IECR = (1UL << AT91C_ID_IRQ0); // 
}  


void irq1_initialize(void) //  IRQ1 ( PA30,  C2)
{
    //   IRQ1
    //AT91C_BASE_PMC->PMC_PCER = (1UL << AT91C_ID_IRQ1) | (1UL << AT91C_ID_PIOA); 
    AT91C_BASE_PMC->PMC_PCER = (1UL << AT91C_ID_IRQ1);
 
    // IRQ  
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA30_IRQ1; //  PA30   PIO
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA30_IRQ1; // I/O    B
    
    //   SYS
    AT91C_BASE_AIC->AIC_IDCR = (1UL << AT91C_ID_IRQ1);  // 
    AT91C_BASE_AIC->AIC_SVR [AT91C_ID_IRQ1] = (AT91_REG) AT91F_IRQ1_Handler; //   
AT91C_BASE_AIC->AIC_SMR [AT91C_ID_IRQ1] = (AT91C_AIC_SRCTYPE & AT91C_AIC_SRCTYPE_POSITIVE_EDGE) | (AT91C_AIC_PRIOR & AT91C_AIC_PRIOR_HIGHEST);
//AT91C_BASE_AIC->AIC_SMR [AT91C_ID_IRQ1] = (AT91C_AIC_SRCTYPE & AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE) | (AT91C_AIC_PRIOR & AT91C_AIC_PRIOR_HIGHEST);
    AT91C_BASE_AIC->AIC_ICCR = (1UL << AT91C_ID_IRQ1); // 
    AT91C_BASE_AIC->AIC_IECR = (1UL << AT91C_ID_IRQ1); // 
}
       	

//static void sensepins_initialize(void) //    PIO ( PA20  PA30)
//{
//   PIOA,   
	//AT91C_BASE_PMC->PMC_PCER = 1UL << AT91C_ID_PIOA; //   
	//AT91C_BASE_PIOA->PIO_ODR = (IPINS);	           //   PA20  PA30   
	//(void) AT91C_BASE_PIOA->PIO_ISR;                 //   	
	//AT91C_BASE_PIOA->PIO_IER = (IPINS);	           //    PA20  PA30
	//AT91C_BASE_PIOA->PIO_IFER = (IPINS);	           //  
	//AT91C_BASE_PIOA->PIO_PPUER = (IPINS);	           // pull-up    PA20  PA30

//   PIO
	//AT91C_BASE_AIC->AIC_IDCR = (1UL << AT91C_ID_PIOA);                          //  
	//AT91C_BASE_AIC->AIC_SVR [AT91C_ID_PIOA] = (AT91_REG) AT91F_PIOA_IRQHandler; // 
	//AT91C_BASE_AIC->AIC_SMR [AT91C_ID_PIOA] = 
		//(AT91C_AIC_SRCTYPE & AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL) |
		//(AT91C_AIC_PRIOR & AT91C_AIC_PRIOR_HIGHEST);
	//AT91C_BASE_AIC->AIC_ICCR = (1UL << AT91C_ID_PIOA);	                       //  
	//AT91C_BASE_AIC->AIC_IECR = (1UL << AT91C_ID_PIOA);	                       //  
//}


void RAMFUNC_NONILINE (prog_val_impl)(unsigned char value, uint_fast8_t n) //     DDS	
{
	uint_fast8_t i;
	unsigned char mask = ((unsigned char) 1) << (n - 1);
	for (i = n; -- i < n; )
	{        
		prog_bit((unsigned char) (value & mask));        
		mask >>= 1;
	}
}


int main(void) //  
{       
	lowlevwl_interrupts_init();                        //  
	usb_disable();                                     // USB	

	AT91C_BASE_PMC->PMC_PCER = (1UL << AT91C_ID_PIOA); //  PIO 	
    
// pull-up 
    AT91C_BASE_PIOA->PIO_PPUDR = SPI_IOUPDATE_BIT;
    AT91C_BASE_PIOA->PIO_PPUDR = SPI_CS_BIT;
    AT91C_BASE_PIOA->PIO_PPUDR = SPI_DATA_BIT;
	AT91C_BASE_PIOA->PIO_PPUDR = SPI_IORESET_BIT;	
	AT91C_BASE_PIOA->PIO_PPUDR = SPI_CLK_BIT;	

//   
    AT91C_BASE_PIOA->PIO_OER = SPI_IOUPDATE_BIT;
    AT91C_BASE_PIOA->PIO_OER = SPI_CS_BIT;
    AT91C_BASE_PIOA->PIO_OER = SPI_DATA_BIT;
	AT91C_BASE_PIOA->PIO_OER = SPI_IORESET_BIT;	
	AT91C_BASE_PIOA->PIO_OER = SPI_CLK_BIT;	
	
//  DDS	
	AT91C_BASE_PIOA->PIO_SODR = SPI_IORESET_BIT;
    _delay_us(10);	
	AT91C_BASE_PIOA->PIO_CODR = SPI_IORESET_BIT;

//  DDS,  I/O_UPDATE    	
    AT91C_BASE_PIOA->PIO_SODR = SPI_CS_BIT;
    AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT;
    
//  DDS
    AT91C_BASE_PIOA->PIO_CODR = SPI_CS_BIT;        

//   (  0)    90 .   0    
    prog_val_impl(0x00, n); //   AD9958   CSR     
    prog_val_impl(0x40, n); //    0
    AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
    _delay_us(1);
    AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //  
    
//   90 .   0
    prog_val_impl(0x05, n); //   AD9958   CPW0
    prog_val_impl(0x10, n); //      AD9958
    prog_val_impl(0x00, n); //      AD9958    
    AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
    _delay_us(1);
    AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //   
    
//  0  1
    prog_val_impl(0x00, n); //   AD9958   CSR     
    prog_val_impl(0xC0, n); //  0  1
    AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
    _delay_us(1);
    AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //   
    
//    (    ,       )   
    //prog_val_impl(0x06, n); //   AD9958   ACR     
    //prog_val_impl(0x00, n); //   ()
    //prog_val_impl(0x13, n); // ,        
    //prog_val_impl(0xFF, n); // ,    
    //AT91C_BASE_PIOA->PIO_SODR = SPI_IOUPDATE_BIT; //    
    //_delay_us(1);
    //AT91C_BASE_PIOA->PIO_CODR = SPI_IOUPDATE_BIT; //           

//       
    //sensepins_initialize();  //    PIO ( PA20  PA30)
    irq0_initialize();         //  IRQ0 ( PA20,  C3)
    irq1_initialize();         //  IRQ1 ( PA30,  C2)
    
    for (;;)
        ;      
    //return 0;
}
